@@ -1,8 +1,16 @@
 /*  Pdlzip - LZMA lossless data compressor
-    Copyright (C) 2010, 2011, 2012, 2013 Antonio Diaz Diaz.
+    Copyright (C) 2010-2015 Antonio Diaz Diaz.
 
-    This program is free software: you have unlimited permission
-    to copy, distribute and modify it.
+    This program is free software. Redistribution and use in source and
+    binary forms, with or without modification, are permitted provided
+    that the following conditions are met:
+
+    1. Redistributions of source code must retain the above copyright
+    notice, this list of conditions and the following disclaimer.
+
+    2. Redistributions in binary form must reproduce the above copyright
+    notice, this list of conditions and the following disclaimer in the
+    documentation and/or other materials provided with the distribution.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -48,12 +56,29 @@ struct Pretty_print
   {
   const char * name;
   const char * stdin_name;
-  int longest_name;
+  unsigned longest_name;
   bool first_post;
   };
 
-void Pp_init( struct Pretty_print * const pp, const char * const filenames[],
-              const int num_filenames );
+static inline void Pp_init( struct Pretty_print * const pp,
+                   const char * const filenames[], const int num_filenames )
+  {
+  unsigned stdin_name_len;
+  int i;
+  pp->name = 0;
+  pp->stdin_name = "(stdin)";
+  pp->longest_name = 0;
+  pp->first_post = false;
+  stdin_name_len = strlen( pp->stdin_name );
+
+  for( i = 0; i < num_filenames; ++i )
+    {
+    const char * const s = filenames[i];
+    const unsigned len = (strcmp( s, "-" ) == 0) ? stdin_name_len : strlen( s );
+    if( len > pp->longest_name ) pp->longest_name = len;
+    }
+  if( pp->longest_name == 0 ) pp->longest_name = stdin_name_len;
+  }
 
 static inline void Pp_set_name( struct Pretty_print * const pp,
                                 const char * const filename )
@@ -86,11 +111,9 @@ static inline void CRC32_init( void )
     }
   }
 
-static inline void CRC32_update_byte( uint32_t * const crc, const uint8_t byte )
-  { *crc = crc32[(*crc^byte)&0xFF] ^ ( *crc >> 8 ); }
-
 static inline void CRC32_update_buf( uint32_t * const crc,
-                                     const uint8_t * const buffer, const int size )
+                                     const uint8_t * const buffer,
+                                     const int size )
   {
   int i;
   for( i = 0; i < size; ++i )
@@ -141,10 +164,10 @@ static inline bool Fh_set_dictionary_size( File_header data, const unsigned sz )
     if( sz > min_dictionary_size )
       {
       const unsigned base_size = 1 << data[5];
-      const unsigned wedge = base_size / 16;
+      const unsigned fraction = base_size / 16;
       int i;
       for( i = 7; i >= 1; --i )
-        if( base_size - ( i * wedge ) >= sz )
+        if( base_size - ( i * fraction ) >= sz )
           { data[5] |= ( i << 5 ); break; }
       }
     return true;
@@ -163,47 +186,35 @@ enum { Ft_size = 20 };
 static inline unsigned Ft_get_data_crc( const File_trailer data )
   {
   unsigned tmp = 0;
-  int i;
-  for( i = 3; i >= 0; --i ) { tmp <<= 8; tmp += data[i]; }
+  int i; for( i = 3; i >= 0; --i ) { tmp <<= 8; tmp += data[i]; }
   return tmp;
   }
 
 static inline void Ft_set_data_crc( File_trailer data, unsigned crc )
-  {
-  int i;
-  for( i = 0; i <= 3; ++i ) { data[i] = (uint8_t)crc; crc >>= 8; }
-  }
+  { int i; for( i = 0; i <= 3; ++i ) { data[i] = (uint8_t)crc; crc >>= 8; } }
 
 static inline unsigned long long Ft_get_data_size( const File_trailer data )
   {
   unsigned long long tmp = 0;
-  int i;
-  for( i = 11; i >= 4; --i ) { tmp <<= 8; tmp += data[i]; }
+  int i; for( i = 11; i >= 4; --i ) { tmp <<= 8; tmp += data[i]; }
   return tmp;
   }
 
 static inline void Ft_set_data_size( File_trailer data, unsigned long long sz )
-  {
-  int i;
-  for( i = 4; i <= 11; ++i ) { data[i] = (uint8_t)sz; sz >>= 8; }
-  }
+  { int i; for( i = 4; i <= 11; ++i ) { data[i] = (uint8_t)sz; sz >>= 8; } }
 
 static inline unsigned long long Ft_get_member_size( const File_trailer data )
   {
   unsigned long long tmp = 0;
-  int i;
-  for( i = 19; i >= 12; --i ) { tmp <<= 8; tmp += data[i]; }
+  int i; for( i = 19; i >= 12; --i ) { tmp <<= 8; tmp += data[i]; }
   return tmp;
   }
 
 static inline void Ft_set_member_size( File_trailer data, unsigned long long sz )
-  {
-  int i;
-  for( i = 12; i <= 19; ++i ) { data[i] = (uint8_t)sz; sz >>= 8; }
-  }
+  { int i; for( i = 12; i <= 19; ++i ) { data[i] = (uint8_t)sz; sz >>= 8; } }
 
 
-/* defined in LzmaDec.c */
+/* defined in main.c */
 int readblock( const int fd, uint8_t * const buf, const int size );
 int writeblock( const int fd, const uint8_t * const buf, const int size );
 
