@@ -1,6 +1,6 @@
 #! /bin/sh
 # check script for Pdlzip - LZMA lossless data compressor
-# Copyright (C) 2010, 2011, 2012, 2013 Antonio Diaz Diaz.
+# Copyright (C) 2010-2015 Antonio Diaz Diaz.
 #
 # This script is free software: you have unlimited permission
 # to copy, distribute and modify it.
@@ -12,7 +12,7 @@ testdir=`cd "$1" ; pwd`
 LZIP="${objdir}"/pdlzip
 framework_failure() { echo "failure in testing framework" ; exit 1 ; }
 
-if [ ! -x "${LZIP}" ] ; then
+if [ ! -f "${LZIP}" ] || [ ! -x "${LZIP}" ] ; then
 	echo "${LZIP}: cannot execute"
 	exit 1
 fi
@@ -27,22 +27,29 @@ fail=0
 
 printf "testing pdlzip-%s..." "$2"
 
-"${LZIP}" -cqs-1 in > /dev/null
-if [ $? = 1 ] ; then printf . ; else fail=1 ; printf - ; fi
-"${LZIP}" -cqs0 in > /dev/null
-if [ $? = 1 ] ; then printf . ; else fail=1 ; printf - ; fi
-"${LZIP}" -cqs4095 in > /dev/null
-if [ $? = 1 ] ; then printf . ; else fail=1 ; printf - ; fi
+"${LZIP}" -cqm4 in > /dev/null
+if [ $? = 1 ] ; then printf . ; else printf - ; fail=1 ; fi
 "${LZIP}" -cqm274 in > /dev/null
-if [ $? = 1 ] ; then printf . ; else fail=1 ; printf - ; fi
-"${LZIP}" -tq in
-if [ $? = 2 ] ; then printf . ; else fail=1 ; printf - ; fi
-"${LZIP}" -tq < in
-if [ $? = 2 ] ; then printf . ; else fail=1 ; printf - ; fi
+if [ $? = 1 ] ; then printf . ; else printf - ; fail=1 ; fi
+"${LZIP}" -cqs-1 in > /dev/null
+if [ $? = 1 ] ; then printf . ; else printf - ; fail=1 ; fi
+"${LZIP}" -cqs0 in > /dev/null
+if [ $? = 1 ] ; then printf . ; else printf - ; fail=1 ; fi
+"${LZIP}" -cqs4095 in > /dev/null
+if [ $? = 1 ] ; then printf . ; else printf - ; fail=1 ; fi
+"${LZIP}" -cqs513MiB in > /dev/null
+if [ $? = 1 ] ; then printf . ; else printf - ; fail=1 ; fi
+printf "  in: Bad magic number (file not in lzip format).\n" > msg
+"${LZIP}" -t in 2> out
+if [ $? = 2 ] && cmp out msg ; then printf . ; else printf - ; fail=1 ; fi
+printf "  (stdin): Bad magic number (file not in lzip format).\n" > msg
+"${LZIP}" -t < in 2> out
+if [ $? = 2 ] && cmp out msg ; then printf . ; else printf - ; fail=1 ; fi
+rm -f out msg
 "${LZIP}" -cdq in
-if [ $? = 2 ] ; then printf . ; else fail=1 ; printf - ; fi
+if [ $? = 2 ] ; then printf . ; else printf - ; fail=1 ; fi
 "${LZIP}" -cdq < in
-if [ $? = 2 ] ; then printf . ; else fail=1 ; printf - ; fi
+if [ $? = 2 ] ; then printf . ; else printf - ; fail=1 ; fi
 dd if="${in_lz}" bs=1 count=6 2> /dev/null | "${LZIP}" -tq
 if [ $? = 2 ] ; then printf . ; else printf - ; fail=1 ; fi
 dd if="${in_lz}" bs=1 count=20 2> /dev/null | "${LZIP}" -tq
@@ -53,20 +60,50 @@ if [ $? = 2 ] ; then printf . ; else printf - ; fail=1 ; fi
 cmp in copy || fail=1
 printf .
 
-"${LZIP}" -cfq "${in_lz}" > out
-if [ $? = 1 ] ; then printf . ; else fail=1 ; printf - ; fi
-"${LZIP}" -cF "${in_lz}" > out || fail=1
-"${LZIP}" -cd out | "${LZIP}" -d > copy || fail=1
-cmp in copy || fail=1
-printf .
-
 "${LZIP}" -t "${testdir}"/test.txt.lzma || fail=1
 "${LZIP}" -cd "${testdir}"/test.txt.lzma > copy || fail=1
 cmp in copy || fail=1
 printf .
 
-for i in s4Ki 0 1 2 3 4 5 6 7 8s16 9s16 ; do
-	"${LZIP}" -k -$i in || fail=1
+cat "${in_lz}" > copy.lz || framework_failure
+printf "to be overwritten" > copy || framework_failure
+"${LZIP}" -df copy.lz || fail=1
+cmp in copy || fail=1
+printf .
+
+printf "to be overwritten" > copy || framework_failure
+"${LZIP}" -df -o copy < "${in_lz}" || fail=1
+cmp in copy || fail=1
+printf .
+
+"${LZIP}" -s16 < in > anyothername || fail=1
+"${LZIP}" -d anyothername || fail=1
+cmp in anyothername.out || fail=1
+printf .
+
+cat in in > in2 || framework_failure
+"${LZIP}" -s16 -o copy2 < in2 || fail=1
+"${LZIP}" -t copy2.lz || fail=1
+printf .
+"${LZIP}" -cd copy2.lz > copy2 || fail=1
+cmp in2 copy2 || fail=1
+printf .
+
+printf "garbage" >> copy2.lz || framework_failure
+printf "to be overwritten" > copy2 || framework_failure
+"${LZIP}" -df copy2.lz || fail=1
+cmp in2 copy2 || fail=1
+printf .
+
+"${LZIP}" -cfq "${in_lz}" > out
+if [ $? = 1 ] ; then printf . ; else printf - ; fail=1 ; fi
+"${LZIP}" -cF -s16 "${in_lz}" > out || fail=1
+"${LZIP}" -cd out | "${LZIP}" -d > copy || fail=1
+cmp in copy || fail=1
+printf .
+
+for i in s4Ki 0 1 2 3 4 5 6 7 8 9 ; do
+	"${LZIP}" -k -$i -s16 in || fail=1
 	mv -f in.lz copy.lz || fail=1
 	printf "garbage" >> copy.lz || fail=1
 	"${LZIP}" -df copy.lz || fail=1
@@ -74,39 +111,28 @@ for i in s4Ki 0 1 2 3 4 5 6 7 8s16 9s16 ; do
 done
 printf .
 
-for i in s4Ki 0 1 2 3 4 5 6 7 8s16 9s16 ; do
-	"${LZIP}" -c -$i in > out || fail=1
+for i in s4Ki 0 1 2 3 4 5 6 7 8 9 ; do
+	"${LZIP}" -c -$i -s16 in > out || fail=1
 	printf "g" >> out || fail=1
 	"${LZIP}" -cd out > copy || fail=1
 	cmp in copy || fail=1
 done
 printf .
 
-for i in s4Ki 0 1 2 3 4 5 6 7 8s16 9s16 ; do
-	"${LZIP}" -$i < in > out || fail=1
+for i in s4Ki 0 1 2 3 4 5 6 7 8 9 ; do
+	"${LZIP}" -$i -s16 < in > out || fail=1
 	"${LZIP}" -d < out > copy || fail=1
 	cmp in copy || fail=1
 done
 printf .
 
-for i in s4Ki 0 1 2 3 4 5 6 7 8s16 9s16 ; do
-	"${LZIP}" -f -$i -o out < in || fail=1
+for i in s4Ki 0 1 2 3 4 5 6 7 8 9 ; do
+	"${LZIP}" -f -$i -s16 -o out < in || fail=1
 	"${LZIP}" -df -o copy < out.lz || fail=1
 	cmp in copy || fail=1
 done
 printf .
 
-"${LZIP}" < in > anyothername || fail=1
-"${LZIP}" -d anyothername || fail=1
-cmp in anyothername.out || fail=1
-printf .
-
-cat in in > in2 || framework_failure
-"${LZIP}" < in2 > out2 || fail=1
-"${LZIP}" -d < out2 > copy2 || fail=1
-cmp in2 copy2 || fail=1
-printf .
-
 echo
 if [ ${fail} = 0 ] ; then
 	echo "tests completed successfully."
